#!/usr/bin/env python
import pytest

import sklearn.datasets as datasets
import sklearn.decomposition as decomposition

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestDecomposition(tm.TestCase):

    def test_objectmapper(self):
        df = pdml.ModelFrame([])
        self.assertIs(df.decomposition.PCA, decomposition.PCA)
        self.assertIs(df.decomposition.IncrementalPCA,
                      decomposition.IncrementalPCA)
        self.assertIs(df.decomposition.KernelPCA, decomposition.KernelPCA)
        self.assertIs(df.decomposition.FactorAnalysis,
                      decomposition.FactorAnalysis)
        self.assertIs(df.decomposition.FastICA, decomposition.FastICA)
        self.assertIs(df.decomposition.TruncatedSVD, decomposition.TruncatedSVD)
        self.assertIs(df.decomposition.NMF, decomposition.NMF)
        self.assertIs(df.decomposition.SparsePCA, decomposition.SparsePCA)
        self.assertIs(df.decomposition.MiniBatchSparsePCA,
                      decomposition.MiniBatchSparsePCA)
        self.assertIs(df.decomposition.SparseCoder, decomposition.SparseCoder)
        self.assertIs(df.decomposition.DictionaryLearning,
                      decomposition.DictionaryLearning)
        self.assertIs(df.decomposition.MiniBatchDictionaryLearning,
                      decomposition.MiniBatchDictionaryLearning)

        self.assertIs(df.decomposition.LatentDirichletAllocation,
                      decomposition.LatentDirichletAllocation)

    def test_fastica(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        result = df.decomposition.fastica(random_state=self.random_state)
        expected = decomposition.fastica(iris.data,
                                         random_state=self.random_state)

        self.assertEqual(len(result), 3)
        self.assertIsInstance(result[0], pdml.ModelFrame)
        tm.assert_index_equal(result[0].index, df.data.columns)
        self.assert_numpy_array_almost_equal(result[0].values, expected[0])

        self.assertIsInstance(result[1], pdml.ModelFrame)
        self.assert_numpy_array_almost_equal(result[1].values, expected[1])

        self.assertIsInstance(result[2], pdml.ModelFrame)
        tm.assert_index_equal(result[2].index, df.index)
        self.assert_numpy_array_almost_equal(result[2].values, expected[2])

        result = df.decomposition.fastica(return_X_mean=True,
                                          random_state=self.random_state)
        expected = decomposition.fastica(iris.data, return_X_mean=True,
                                         random_state=self.random_state)

        self.assertEqual(len(result), 4)
        self.assertIsInstance(result[0], pdml.ModelFrame)
        tm.assert_index_equal(result[0].index, df.data.columns)
        self.assert_numpy_array_almost_equal(result[0].values, expected[0])

        self.assertIsInstance(result[1], pdml.ModelFrame)
        self.assert_numpy_array_almost_equal(result[1].values, expected[1])

        self.assertIsInstance(result[2], pdml.ModelFrame)
        tm.assert_index_equal(result[2].index, df.index)
        self.assert_numpy_array_almost_equal(result[2].values, expected[2])

        self.assert_numpy_array_almost_equal(result[3], expected[3])

    def test_dict_learning(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        result = df.decomposition.dict_learning(2, 1, random_state=self.random_state)
        expected = decomposition.dict_learning(iris.data, 2, 1,
                                               random_state=self.random_state)
        self.assertEqual(len(result), 3)
        self.assertIsInstance(result[0], pdml.ModelFrame)
        tm.assert_index_equal(result[0].index, df.data.index)
        self.assert_numpy_array_almost_equal(result[0].values, expected[0])

        self.assertIsInstance(result[1], pdml.ModelFrame)
        tm.assert_index_equal(result[1].columns, df.data.columns)
        self.assert_numpy_array_almost_equal(result[1].values, expected[1])

        self.assert_numpy_array_almost_equal(result[2], expected[2])

    def test_dict_learning_online(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        result = df.decomposition.dict_learning_online(random_state=self.random_state)
        expected = decomposition.dict_learning_online(iris.data,
                                                      random_state=self.random_state)

        self.assertEqual(len(result), 2)
        self.assertIsInstance(result[0], pdml.ModelFrame)
        tm.assert_index_equal(result[0].index, df.data.index)
        self.assert_numpy_array_almost_equal(result[0].values, expected[0])

        self.assertIsInstance(result[1], pdml.ModelFrame)
        tm.assert_index_equal(result[1].columns, df.data.columns)
        self.assert_numpy_array_almost_equal(result[1].values, expected[1])

        result = df.decomposition.dict_learning_online(return_code=False,
                                                       random_state=self.random_state)
        expected = decomposition.dict_learning_online(iris.data,
                                                      return_code=False,
                                                      random_state=self.random_state)
        self.assertIsInstance(result, pdml.ModelFrame)
        tm.assert_index_equal(result.columns, df.data.columns)
        self.assert_numpy_array_almost_equal(result.values, expected)

    def test_sparse_encode(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        _, dictionary, _ = decomposition.dict_learning(iris.data, 2, 1,
                                                       random_state=self.random_state)

        result = df.decomposition.sparse_encode(dictionary)
        expected = decomposition.sparse_encode(iris.data, dictionary)
        self.assertIsInstance(result, pdml.ModelFrame)
        tm.assert_index_equal(result.index, df.data.index)
        self.assert_numpy_array_almost_equal(result.values, expected)

    def test_Decompositions_PCA(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        mod1 = df.decomposition.PCA()
        mod2 = decomposition.PCA()

        df.fit(mod1)
        mod2.fit(iris.data, iris.target)

        result = df.transform(mod1)
        expected = mod2.transform(iris.data)

        self.assertIsInstance(result, pdml.ModelFrame)
        tm.assert_series_equal(df.target, result.target)
        self.assert_numpy_array_almost_equal(result.data.values,
                                             expected)

    def test_fit_transform_PCA(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        mod1 = df.decomposition.PCA()
        mod2 = decomposition.PCA()

        result = df.fit_transform(mod1)
        expected = mod2.fit_transform(iris.data, iris.target)

        self.assertIsInstance(result, pdml.ModelFrame)
        tm.assert_series_equal(df.target, result.target)
        self.assert_numpy_array_almost_equal(result.data.values,
                                             expected)

    def test_Decompositions_KernelPCA(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        mod1 = df.decomposition.KernelPCA()
        mod2 = decomposition.KernelPCA()

        df.fit(mod1)
        mod2.fit(iris.data, iris.target)

        result = df.transform(mod1)
        expected = mod2.transform(iris.data)

        self.assertIsInstance(result, pdml.ModelFrame)
        tm.assert_series_equal(df.target, result.target)
        self.assert_numpy_array_almost_equal(result.data.values[:, :40],
                                             expected[:, :40])

    def test_fit_transform_KernelPCA(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        mod1 = df.decomposition.KernelPCA()
        mod2 = decomposition.KernelPCA()

        result = df.fit_transform(mod1)
        expected = mod2.fit_transform(iris.data, iris.target)

        self.assertIsInstance(result, pdml.ModelFrame)
        tm.assert_series_equal(df.target, result.target)
        self.assert_numpy_array_almost_equal(result.data.values[:, :40],
                                             expected[:, :40])

    @pytest.mark.parametrize("algo", ['PCA'])
    def test_inverse_transform(self, algo):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        mod1 = getattr(df.decomposition, algo)()
        mod2 = getattr(decomposition, algo)()

        df.fit(mod1)
        mod2.fit(iris.data, iris.target)

        result = df.transform(mod1)
        expected = mod2.transform(iris.data)

        self.assertIsInstance(result, pdml.ModelFrame)
        tm.assert_series_equal(df.target, result.target)
        self.assert_numpy_array_almost_equal(result.data.values, expected)

        result = df.inverse_transform(mod1)
        expected = mod2.inverse_transform(iris.data)

        self.assertIsInstance(result, pdml.ModelFrame)
        tm.assert_series_equal(df.target, result.target)
        self.assert_numpy_array_almost_equal(result.data.values, expected)
        tm.assert_index_equal(result.columns, df.columns)



#!/usr/bin/env python

import numpy as np
import pandas as pd

import sklearn.datasets as datasets

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestModelFrameGroupBy(tm.TestCase):

    def test_frame_groupby(self):
        df = pd.DataFrame({'A': [1, 2, 1, 2],
                           'B': [4, 5, 6, 7],
                           'C': [7, 8, 9, 10]},
                          columns=['A', 'B', 'C'])
        s = pd.Series([1, 2, 3, 4])

        mdf = pdml.ModelFrame(df, target=s)
        self.assertIsInstance(mdf, pdml.ModelFrame)

        grouped = mdf.groupby('A')
        self.assertIsInstance(grouped, pdml.core.groupby.ModelFrameGroupBy)

        df = grouped.get_group(1)
        self.assertIsInstance(df, pdml.ModelFrame)

        expected = pd.Series([1, 3], index=[0, 2], name='.target')
        tm.assert_series_equal(df.target, expected)
        self.assertIsInstance(df.target, pdml.ModelSeries)

    def test_transform_standard(self):
        # check pandas standard transform works

        df = pd.DataFrame({'A': ['A', 'B', 'A', 'A', 'A', 'B', 'B', 'B'],
                           'B': np.random.randn(8),
                           'C': np.random.randn(8)})

        mdf = pdml.ModelFrame(df)
        tm.assert_frame_equal(df.groupby('A').transform('mean'),
                              mdf.groupby('A').transform('mean'))

    def test_grouped_estimator_SVC(self):
        df = pdml.ModelFrame(datasets.load_iris())
        df['sepal length (cm)'] = df['sepal length (cm)'].pp.binarize(threshold=5.8)
        grouped = df.groupby('sepal length (cm)')
        self.assertIsInstance(grouped, pdml.core.groupby.ModelFrameGroupBy)
        for name, group in grouped:
            self.assertIsInstance(group, pdml.ModelFrame)
            self.assertEqual(group.target_name, '.target')
            self.assertTrue(group.has_target())
            tm.assert_index_equal(group.columns, df.columns)
        svc = df.svm.SVC(random_state=self.random_state)
        gclf = grouped.fit(svc)
        self.assertIsInstance(gclf, pdml.core.groupby.GroupedEstimator)
        self.assertEqual(len(gclf.groups), 2)

        results = grouped.predict(gclf)
        self.assertIsInstance(results, pdml.core.groupby.ModelSeriesGroupBy)
        self.assertIsInstance(results.get_group(0), pdml.ModelSeries)
        self.assertIsInstance(results.get_group(1), pdml.ModelSeries)
        # test indexes are preserved
        tm.assert_index_equal(results.get_group(0).index, grouped.get_group(0).index)
        tm.assert_index_equal(results.get_group(1).index, grouped.get_group(1).index)

        import sklearn.svm as svm
        svc1 = svm.SVC(random_state=self.random_state)
        svc2 = svm.SVC(random_state=self.random_state)

        group1 = df[df['sepal length (cm)'] == 0]
        svc1.fit(group1.data.values, group1.target.values)
        expected1 = svc1.predict(group1.data.values)

        group2 = df[df['sepal length (cm)'] == 1]
        svc2.fit(group2.data.values, group2.target.values)
        expected2 = svc2.predict(group2.data.values)

        tm.assert_numpy_array_equal(results.get_group(0).values, expected1)
        tm.assert_numpy_array_equal(results.get_group(1).values, expected2)

    def test_grouped_estimator_PCA(self):
        df = pdml.ModelFrame(datasets.load_iris())
        grouped = df.groupby('.target')
        self.assertIsInstance(grouped, pdml.core.groupby.ModelFrameGroupBy)
        for name, group in grouped:
            self.assertIsInstance(group, pdml.ModelFrame)
            self.assertEqual(group.target_name, '.target')
            self.assertTrue(group.has_target())
            tm.assert_index_equal(group.columns, df.columns)
        pca = df.decomposition.PCA()
        gclf = grouped.fit(pca)
        self.assertIsInstance(gclf, pdml.core.groupby.GroupedEstimator)
        self.assertEqual(len(gclf.groups), 3)

        results = grouped.transform(gclf)
        self.assertIsInstance(results, pdml.core.groupby.ModelFrameGroupBy)

        self.assertIsInstance(results.get_group(0), pdml.ModelFrame)
        self.assertIsInstance(results.get_group(1), pdml.ModelFrame)
        self.assertIsInstance(results.get_group(2), pdml.ModelFrame)
        # test indexes are preserved
        tm.assert_index_equal(results.get_group(0).index, grouped.get_group(0).index)
        tm.assert_index_equal(results.get_group(1).index, grouped.get_group(1).index)
        tm.assert_index_equal(results.get_group(2).index, grouped.get_group(2).index)

        import sklearn.decomposition as dc
        for i in range(3):
            group = df[df['.target'] == i]
            pca = dc.PCA()
            pca.fit(group.data.values)
            expected = pca.transform(group.data.values)
            result = results.get_group(i).data
            self.assert_numpy_array_almost_equal(result.values, expected)


class TestModelSeriesGroupBy(tm.TestCase):

    def test_series_groupby(self):
        s = pdml.ModelSeries([1, 2, 1, 2], name='X')
        self.assertIsInstance(s, pdml.ModelSeries)

        grouped = s.groupby([1, 1, 1, 2])
        self.assertIsInstance(grouped, pdml.core.groupby.ModelSeriesGroupBy)

        gs = grouped.get_group(1)
        self.assertIsInstance(gs, pdml.ModelSeries)
        expected = pd.Series([1, 2, 1], index=[0, 1, 2], name='X')
        tm.assert_series_equal(gs, expected)
        self.assertEqual(gs.name, 'X')



#!/usr/bin/env python


def _is_1d_varray(arr):
    """
    Check whether the array has single dimension like (x, ) or (x, 1)
    """
    return len(arr.shape) < 2 or arr.shape[1] == 1


def _is_1d_harray(arr):
    """
    Check whether the array has single dimension like (x, ) or (1, x)
    """
    return len(arr.shape) < 2 or arr.shape[0] == 1



#!/usr/bin/env python

from pandas_ml.core.accessor import (_AccessorMethods, _attach_methods,
                                     _wrap_target_pred_func,
                                     _wrap_target_pred_noargs)
import pandas_ml.util as util


class MetricsMethods(_AccessorMethods):
    """
    Accessor to ``sklearn.metrics``.
    """

    _module_name = 'sklearn.metrics'

    # Model Selection Interface
    # ``make_scorer`` will be attached via autoload

    # Clasification metrics

    def auc(self, kind='roc', reorder=False, **kwargs):
        """
        Calcurate AUC of ROC curve or precision recall curve

        Parameters
        ----------
        kind : {'roc', 'precision_recall_curve'}

        Returns
        -------
        float : AUC
        """
        if kind == 'roc':
            return self.roc_auc_score(**kwargs)
        elif kind == 'precision_recall_curve':
            return self.average_precision_score(**kwargs)
        else:
            msg = "Invalid kind: {0}, kind must be either 'roc' or 'precision_recall_curve'"
            raise ValueError(msg.format(kind))

    def average_precision_score(self, *args, **kwargs):
        """
        Call ``sklearn.metrics.average_precision_score`` using automatic mapping.

        - ``y_true``: ``ModelFrame.target``
        - ``y_score``: ``ModelFrame.decision``
        """
        func = self._module.average_precision_score
        return self._score_wraps(func, self._decision, *args, **kwargs)

    def confusion_matrix(self, *args, **kwargs):
        """
        Call ``sklearn.metrics.confusion_matrix`` using automatic mapping.

        - ``y_true``: ``ModelFrame.target``
        - ``y_pred``: ``ModelFrame.predicted``
        """
        func = self._module.confusion_matrix
        result = func(self._target.values, self._predicted.values,
                      *args, **kwargs)
        result = self._constructor(result)
        result.index.name = 'Target'
        result.columns.name = 'Predicted'
        return result

    def f1_score(self, *args, **kwargs):
        """
        Call ``sklearn.metrics.f1_score`` using automatic mapping.

        - ``y_true``: ``ModelFrame.target``
        - ``y_pred``: ``ModelFrame.predicted``
        """
        func = self._module.f1_score
        return self._score_wraps(func, self._predicted, *args, **kwargs)

    def fbeta_score(self, beta, *args, **kwargs):
        """
        Call ``sklearn.metrics.fbeta_score`` using automatic mapping.

        - ``y_true``: ``ModelFrame.target``
        - ``y_pred``: ``ModelFrame.predicted``
        """
        func = self._module.fbeta_score
        return self._score_wraps(func, self._predicted, beta, *args, **kwargs)

    def _score_wraps(self, func, scorerer, *args, **kwargs):
        average = kwargs.get('average', 'weighted')
        result = func(self._target.values, scorerer.values, *args, **kwargs)
        if average is None:
            result = self._constructor_sliced(result)
        return result

    def hinge_loss(self, *args, **kwargs):
        """
        Call ``sklearn.metrics.hinge_loss`` using automatic mapping.

        - ``y_true``: ``ModelFrame.target``
        - ``y_pred_decision``: ``ModelFrame.decision``
        """
        func = self._module.hinge_loss
        result = func(self._target.values,
                      self._df.decision.values, *args, **kwargs)
        return result

    def log_loss(self, *args, **kwargs):
        """
        Call ``sklearn.metrics.log_loss`` using automatic mapping.

        - ``y_true``: ``ModelFrame.target``
        - ``y_pred``: ``ModelFrame.proba``
        """
        func = self._module.log_loss
        result = func(self._target.values,
                      self._df.proba.values, *args, **kwargs)
        return result

    def precision_recall_curve(self, *args, **kwargs):
        """
        Call ``sklearn.metrics.precision_recall_curve`` using automatic mapping.

        - ``y_true``: ``ModelFrame.target``
        - ``y_probas_pred``: ``ModelFrame.decision``
        """
        func = self._module.precision_recall_curve
        return self._curve_wraps(func, *args, **kwargs)

    def _curve_wraps(self, func, *args, **kwargs):
        decision = self._df.decision
        if util._is_1d_varray(decision):
            c1, c2, threshold = func(self._target.values, decision.values,
                                     *args, **kwargs)
            return c1, c2, threshold

        results = {}
        if self._df.has_multi_targets():
            for i, ((name, col), t) in enumerate(zip(decision.iteritems(), self._target.values.T)):
                # results can have different length
                c1, c2, threshold = func(t, col.values, pos_label=i, *args, **kwargs)
                results[name] = c1, c2, threshold
        else:
            for i, (name, col) in enumerate(decision.iteritems()):
                # results can have different length
                c1, c2, threshold = func(self._target.values, col.values,
                                         pos_label=i, *args, **kwargs)
                results[name] = c1, c2, threshold
        return results

    def precision_recall_fscore_support(self, *args, **kwargs):
        """
        Call ``sklearn.metrics.precision_recall_fscore_support`` using automatic mapping.

        - ``y_true``: ``ModelFrame.target``
        - ``y_pred``: ``ModelFrame.predicted``
        """
        func = self._module.precision_recall_fscore_support
        p, r, f, s = func(self._target.values, self._predicted.values,
                          *args, **kwargs)
        result = self._constructor({'precision': p, 'recall': r,
                                    'f1-score': f, 'support': s},
                                   columns=['precision', 'recall', 'f1-score', 'support'])
        return result

    def precision_score(self, *args, **kwargs):
        """
        Call ``sklearn.metrics.precision_score`` using automatic mapping.

        - ``y_true``: ``ModelFrame.target``
        - ``y_pred``: ``ModelFrame.predicted``
        """
        func = self._module.precision_score
        return self._score_wraps(func, self._predicted, *args, **kwargs)

    def recall_score(self, *args, **kwargs):
        """
        Call ``sklearn.metrics.recall_score`` using automatic mapping.

        - ``y_true``: ``ModelFrame.target``
        - ``y_true``: ``ModelFrame.predicted``
        """
        func = self._module.recall_score
        return self._score_wraps(func, self._predicted, *args, **kwargs)

    def roc_auc_score(self, *args, **kwargs):
        """
        Call ``sklearn.metrics.roc_auc_score`` using automatic mapping.

        - ``y_true``: ``ModelFrame.target``
        - ``y_score``: ``ModelFrame.decision``
        """
        func = self._module.roc_auc_score
        return self._score_wraps(func, self._decision, *args, **kwargs)

    def roc_curve(self, *args, **kwargs):
        """
        Call ``sklearn.metrics.roc_curve`` using automatic mapping.

        - ``y_true``: ``ModelFrame.target``
        - ``y_score``: ``ModelFrame.decision``
        """
        func = self._module.roc_curve
        return self._curve_wraps(func, *args, **kwargs)

    # Regression metrics
    # None

    # Clusteing metrics

    def silhouette_score(self, *args, **kwargs):
        """
        Call ``sklearn.metrics.silhouette_score`` using automatic mapping.

        - ``X``: ``ModelFrame.data``
        - ``labels``: ``ModelFrame.predicted``
        """
        func = self._module.silhouette_score
        result = func(self._data.values, self._predicted.values,
                      *args, **kwargs)
        return result

    def silhouette_samples(self, *args, **kwargs):
        """
        Call ``sklearn.metrics.silhouette_samples`` using automatic mapping.

        - ``X``: ``ModelFrame.data``
        - ``labels``: ``ModelFrame.predicted``
        """
        func = self._module.silhouette_samples
        result = func(self._data.values, self._predicted.values,
                      *args, **kwargs)
        result = self._constructor_sliced(result, index=self._df.index)
        return result

    # Biclustering metrics

    def consensus_score(self, *args, **kwargs):
        """Not implemented"""
        raise NotImplementedError

    # Pairwise metrics

    @property
    def pairwise(self):
        """Not implemented"""
        raise NotImplementedError


# y_true and y_pred
_classification_methods = ['accuracy_score', 'classification_report',
                           'hamming_loss', 'jaccard_similarity_score',
                           'matthews_corrcoef', 'zero_one_loss']
_regression_methods = ['explained_variance_score', 'mean_absolute_error',
                       'mean_squared_error', 'r2_score']
_cluster_methods = ['mutual_info_score']
_true_pred_methods = (_classification_methods + _regression_methods
                      + _cluster_methods)
_attach_methods(MetricsMethods, _wrap_target_pred_func, _true_pred_methods)


# methods which doesn't take additional arguments
_cluster_methods_noargs = ['adjusted_mutual_info_score',
                           'adjusted_rand_score',
                           'completeness_score',
                           'homogeneity_completeness_v_measure',
                           'homogeneity_score',
                           'normalized_mutual_info_score',
                           'v_measure_score']
_attach_methods(MetricsMethods, _wrap_target_pred_noargs, _cluster_methods_noargs)



#!/usr/bin/python
# -*- coding: utf8 -*-
import pytest

import matplotlib
matplotlib.use('Agg')

import numpy as np                       # noqa
import pandas as pd                      # noqa
import pandas_ml.util.testing as tm      # noqa

import pandas_ml as pdml                 # noqa
from pandas_ml import ConfusionMatrix    # noqa
from collections import OrderedDict      # noqa

# =========================================================================


def asserts(y_true, y_pred, cm):
    df = cm.to_dataframe()
    a = cm.to_array()

    df_with_sum = cm.to_dataframe(calc_sum=True)

    assert len(y_true) == len(y_pred)

    assert isinstance(df, pd.DataFrame)
    assert isinstance(a, np.ndarray)
    assert isinstance(df_with_sum, pd.DataFrame)

    N = len(df.index)
    assert N == len(df.columns)
    assert cm.len() == len(df.columns)

    assert df.index.name == 'Actual'
    assert df.columns.name == 'Predicted'

    assert df_with_sum.index.name == 'Actual'
    assert df_with_sum.columns.name == 'Predicted'

    # np.testing.assert_array_equal(confusion_matrix(y_true, y_pred), cm.toarray())

    assert cm.sum() == len(y_true)

    assert cm.true.name == 'Actual'
    assert cm.pred.name == 'Predicted'

# =========================================================================


class TestConfusionMatrix(tm.TestCase):

    def test_pandas_confusion_cm_strings(self):
        y_true = ['rabbit', 'cat', 'rabbit', 'rabbit', 'cat', 'dog', 'dog', 'rabbit', 'rabbit', 'cat', 'dog', 'rabbit']
        y_pred = ['cat', 'cat', 'rabbit', 'dog', 'cat', 'rabbit', 'dog', 'cat', 'rabbit', 'cat', 'rabbit', 'rabbit']
        cm = pdml.ConfusionMatrix(y_true, y_pred)
        assert isinstance(cm, pdml.confusion_matrix.LabeledConfusionMatrix)
        print("Confusion matrix:\n%s" % cm)
        asserts(y_true, y_pred, cm)
        # cm.print_stats()

    def test_pandas_confusion_cm_int(self):
        y_true = [2, 0, 2, 2, 0, 1, 1, 2, 2, 0, 1, 2]
        y_pred = [0, 0, 2, 1, 0, 2, 1, 0, 2, 0, 2, 2]
        labels = ["ant", "bird", "cat"]
        cm = ConfusionMatrix(y_true, y_pred, labels=labels)
        assert isinstance(cm, pdml.confusion_matrix.LabeledConfusionMatrix)
        print("Confusion matrix:\n%s" % cm)
        asserts(y_true, y_pred, cm)
        assert cm.len() == len(labels)
        # np.testing.assert_array_equal(confusion_matrix(y_true, y_pred), cm.toarray())
        # cm.print_stats()

    def test_pandas_confusion_binary_cm(self):
        y_true = [True, True, False, False, False, True, False, True, True,
                  False, True, False, False, False, False, False, True, False,
                  True, True, True, True, False, False, False, True, False,
                  True, False, False, False, False, True, True, False, False,
                  False, True, True, True, True, False, False, False, False,
                  True, False, False, False, False, False, False, False, False,
                  False, True, True, False, True, False, True, True, True,
                  False, False, True, False, True, False, False, True, False,
                  False, False, False, False, False, False, False, True, False,
                  True, True, True, True, False, False, True, False, True,
                  True, False, True, False, True, False, False, True, True,
                  False, False, True, True, False, False, False, False, False,
                  False, True, True, False]

        y_pred = [False, False, False, False, False, True, False, False, True,
                  False, True, False, False, False, False, False, False, False,
                  True, True, True, True, False, False, False, False, False,
                  False, False, False, False, False, True, False, False, False,
                  False, True, False, False, False, False, False, False, False,
                  True, False, False, False, False, False, False, False, False,
                  False, True, False, False, False, False, False, False, False,
                  False, False, True, False, False, False, False, True, False,
                  False, False, False, False, False, False, False, True, False,
                  False, True, False, False, False, False, True, False, True,
                  True, False, False, False, True, False, False, True, True,
                  False, False, True, True, False, False, False, False, False,
                  False, True, False, False]

        binary_cm = ConfusionMatrix(y_true, y_pred)
        assert isinstance(binary_cm, pdml.confusion_matrix.BinaryConfusionMatrix)

        print("Binary confusion matrix:\n%s" % binary_cm)
        asserts(y_true, y_pred, binary_cm)

    def test_pandas_confusion_cm_empty_column(self):
        y_true = [2, 0, 2, 2, 0, 1]
        y_pred = [0, 0, 2, 2, 0, 2]
        # cm = LabeledConfusionMatrix(y_true, y_pred)
        cm = ConfusionMatrix(y_true, y_pred, labels=["ant", "bird", "cat"])
        assert isinstance(cm, pdml.confusion_matrix.LabeledConfusionMatrix)

        cm = ConfusionMatrix(y_true, y_pred)
        assert isinstance(cm, pdml.confusion_matrix.LabeledConfusionMatrix)

        print("Confusion matrix:\n%s" % cm)
        asserts(y_true, y_pred, cm)

        # cm.print_stats()

    def test_pandas_confusion_cm_empty_row(self):
        y_true = [2, 0, 2, 2, 0, 0]
        y_pred = [0, 0, 2, 2, 1, 2]
        # cm = LabeledConfusionMatrix(y_true, y_pred)
        cm = ConfusionMatrix(y_true, y_pred, labels=["ant", "bird", "cat"])
        assert isinstance(cm, pdml.confusion_matrix.LabeledConfusionMatrix)

        cm = ConfusionMatrix(y_true, y_pred)
        assert isinstance(cm, pdml.confusion_matrix.LabeledConfusionMatrix)

        print("Confusion matrix:\n%s" % cm)
        asserts(y_true, y_pred, cm)

        # cm.print_stats()

    def test_pandas_confusion_cm_binarize(self):
        y_true = ['rabbit', 'cat', 'rabbit', 'rabbit', 'cat', 'dog', 'dog', 'rabbit', 'rabbit', 'cat', 'dog', 'rabbit']
        y_pred = ['cat', 'cat', 'rabbit', 'dog', 'cat', 'rabbit', 'dog', 'cat', 'rabbit', 'cat', 'rabbit', 'rabbit']

        cm = ConfusionMatrix(y_true, y_pred)
        assert isinstance(cm, pdml.confusion_matrix.LabeledConfusionMatrix)

        print("Confusion matrix:\n%s" % cm)
        select = ['cat', 'dog']

        print("Binarize with %s" % select)
        binary_cm = cm.binarize(select)

        print("Binary confusion matrix:\n%s" % binary_cm)

        assert cm.sum() == binary_cm.sum()

    def test_value_counts(self):
        df = pd.DataFrame({
            'Height': [150, 150, 151, 151, 152, 155, 155, 157, 157, 157, 157, 158, 158, 159, 159, 159, 160, 160, 162, 162, 163, 164, 165, 168, 169, 169, 169, 170, 171, 171, 173, 173, 174, 176, 177, 177, 179, 179, 179, 179, 179, 181, 181, 182, 183, 184, 186, 190, 190],
            'Weight': [54, 55, 55, 47, 58, 53, 59, 60, 56, 55, 62, 56, 55, 55, 64, 61, 59, 59, 63, 66, 64, 62, 66, 66, 72, 65, 75, 71, 70, 70, 75, 65, 79, 78, 83, 75, 84, 78, 74, 75, 74, 90, 80, 81, 90, 81, 91, 87, 100],
            'Size': ['S', 'S', 'S', 'S', 'S', 'S', 'S', 'S', 'S', 'S', 'S', 'S', 'S', 'S', 'S', 'S', 'S', 'S', 'S', 'M', 'M', 'M', 'M', 'M', 'M', 'M', 'M', 'L', 'L', 'L', 'L', 'L', 'L', 'L', 'L', 'L', 'L', 'L', 'L', 'L', 'L', 'XL', 'XL', 'XL', 'XL', 'XL', 'XL', 'XL', 'XL'],
            'SizePred': ['S', 'S', 'S', 'S', 'S', 'S', 'S', 'S', 'S', 'S', 'S', 'S', 'S', 'S', 'S', 'S', 'S', 'S', 'M', 'M', 'M', 'M', 'M', 'M', 'M', 'M', 'M', 'M', 'M', 'M', 'L', 'M', 'L', 'L', 'L', 'L', 'L', 'L', 'L', 'L', 'L', 'XL', 'L', 'L', 'XL', 'L', 'XL', 'XL', 'XL'],
        })
        cm = ConfusionMatrix(df["Size"], df["SizePred"])
        assert isinstance(cm, pdml.confusion_matrix.LabeledConfusionMatrix)

        assert (cm.true - df.Size.value_counts()).sum() == 0
        assert (cm.pred - df.SizePred.value_counts()).sum() == 0
        cm.print_stats()

    def test_pandas_confusion_cm_stats_integers(self):
        y_true = [600, 200, 200, 200, 200, 200, 200, 200, 500, 500, 500, 200, 200, 200, 200, 200, 200, 200, 200, 200]
        y_pred = [100, 200, 200, 100, 100, 200, 200, 200, 100, 200, 500, 100, 100, 100, 100, 100, 100, 100, 500, 200]
        print("y_true: %s" % y_true)
        print("y_pred: %s" % y_pred)
        cm = ConfusionMatrix(y_true, y_pred)
        assert isinstance(cm, pdml.confusion_matrix.LabeledConfusionMatrix)

        assert isinstance(cm.stats(), OrderedDict)
        cm.print_stats()

    def test_pandas_confusion_cm_stats_animals(self):
        y_true = ['rabbit', 'cat', 'rabbit', 'rabbit', 'cat', 'dog', 'dog', 'rabbit', 'rabbit', 'cat', 'dog', 'rabbit']
        y_pred = ['cat', 'cat', 'rabbit', 'dog', 'cat', 'rabbit', 'dog', 'cat', 'rabbit', 'cat', 'rabbit', 'rabbit']
        print("y_true: %s" % y_true)
        print("y_pred: %s" % y_pred)

        cm = ConfusionMatrix(y_true, y_pred)
        assert isinstance(cm, pdml.confusion_matrix.LabeledConfusionMatrix)

        assert isinstance(cm.stats(), OrderedDict)
        assert cm.population == len(y_true)  # 12
        cm.print_stats()
        cm_stats = cm.stats()  # noqa

        assert cm.binarize("cat").TP == cm.get("cat")  # cm.get("cat", "cat")
        assert cm.binarize("cat").TP == 3
        assert cm.binarize("dog").TP == cm.get("dog")  # 1
        assert cm.binarize("rabbit").TP == cm.get("rabbit")  # 3
        # print cm.TP

    def test_pandas_confusion_get(self):
        y_true = ['rabbit', 'cat', 'rabbit', 'rabbit', 'cat', 'dog', 'dog', 'rabbit', 'rabbit', 'cat', 'dog', 'rabbit']
        y_pred = ['cat', 'cat', 'rabbit', 'dog', 'cat', 'rabbit', 'dog', 'cat', 'rabbit', 'cat', 'rabbit', 'rabbit']
        print("y_true: %s" % y_true)
        print("y_pred: %s" % y_pred)

        cm = ConfusionMatrix(y_true, y_pred)
        assert isinstance(cm, pdml.confusion_matrix.LabeledConfusionMatrix)

        assert cm.get("cat") == cm.get("cat", "cat")
        assert cm.get("cat") == 3
        assert cm.get("dog") == 1
        assert cm.get("rabbit") == 3
        assert cm.get("dog", "rabbit") == 2

    def test_pandas_confusion_max_min(self):
        y_true = ['rabbit', 'cat', 'rabbit', 'rabbit', 'cat', 'dog', 'dog', 'rabbit', 'rabbit', 'cat', 'dog', 'rabbit']
        y_pred = ['cat', 'cat', 'rabbit', 'dog', 'cat', 'rabbit', 'dog', 'cat', 'rabbit', 'cat', 'rabbit', 'rabbit']
        print("y_true: %s" % y_true)
        print("y_pred: %s" % y_pred)

        cm = ConfusionMatrix(y_true, y_pred)
        assert isinstance(cm, pdml.confusion_matrix.LabeledConfusionMatrix)

        assert cm.max() == 3
        assert cm.min() == 0

    def test_pandas_confusion_binary_cm_inverse(self):
        y_true = [True, True, False, False, False, True, False, True, True,
                  False, True, False, False, False, False, False, True, False,
                  True, True, True, True, False, False, False, True, False,
                  True, False, False, False, False, True, True, False, False,
                  False, True, True, True, True, False, False, False, False,
                  True, False, False, False, False, False, False, False, False,
                  False, True, True, False, True, False, True, True, True,
                  False, False, True, False, True, False, False, True, False,
                  False, False, False, False, False, False, False, True, False,
                  True, True, True, True, False, False, True, False, True,
                  True, False, True, False, True, False, False, True, True,
                  False, False, True, True, False, False, False, False, False,
                  False, True, True, False]

        y_pred = [False, False, False, False, False, True, False, False, True,
                  False, True, False, False, False, False, False, False, False,
                  True, True, True, True, False, False, False, False, False,
                  False, False, False, False, False, True, False, False, False,
                  False, True, False, False, False, False, False, False, False,
                  True, False, False, False, False, False, False, False, False,
                  False, True, False, False, False, False, False, False, False,
                  False, False, True, False, False, False, False, True, False,
                  False, False, False, False, False, False, False, True, False,
                  False, True, False, False, False, False, True, False, True,
                  True, False, False, False, True, False, False, True, True,
                  False, False, True, True, False, False, False, False, False,
                  False, True, False, False]

        binary_cm = ConfusionMatrix(y_true, y_pred)
        assert isinstance(binary_cm, pdml.confusion_matrix.BinaryConfusionMatrix)
        bcm_sum = binary_cm.sum()

        binary_cm_r = binary_cm.inverse()  # reverse not in place
        assert bcm_sum == binary_cm_r.sum()

    """
    def test_enlarge_confusion_matrix():
        #cm.enlarge(300)
        #cm.enlarge([300, 400])

    def test_pandas_confusion_binarize():
        y_true = [600, 200, 200, 200, 200, 200, 200, 200, 500, 500, 500, 200, 200, 200, 200, 200, 200, 200, 200, 200]
        y_pred = [100, 200, 200, 100, 100, 200, 200, 200, 100, 200, 500, 100, 100, 100, 100, 100, 100, 100, 500, 200]
        cm = LabeledConfusionMatrix(y_true, y_pred)
        binary_cm_100 = cm.binarize(100)
        print("\n%s" % binary_cm_100)
    """

    def test_pandas_confusion_normalized(self):
        y_true = [2, 0, 2, 2, 0, 1, 1, 2, 2, 0, 1, 2]
        y_pred = [0, 0, 2, 1, 0, 2, 1, 0, 2, 0, 2, 2]
        cm = ConfusionMatrix(y_true, y_pred)
        assert isinstance(cm, pdml.confusion_matrix.LabeledConfusionMatrix)

        df = cm.to_dataframe()
        df_norm = cm.to_dataframe(normalized=True)
        assert(df_norm.sum(axis=1).sum() == len(df))

    def test_pandas_confusion_normalized_issue1(self):
        # should insure issue 1 is fixed
        # see http://stackoverflow.com/questions/19233771/sklearn-plot-confusion-matrix-with-labels/31720054#31720054

        y_true = ['business', 'business', 'business', 'business', 'business',
                  'business', 'business', 'business', 'business', 'business',
                  'business', 'business', 'business', 'business', 'business',
                  'business', 'business', 'business', 'business', 'business']

        y_pred = ['health', 'business', 'business', 'business', 'business',
                  'business', 'health', 'health', 'business', 'business', 'business',
                  'business', 'business', 'business', 'business', 'business',
                  'health', 'health', 'business', 'health']

        cm = ConfusionMatrix(y_true, y_pred)
        assert isinstance(cm, pdml.confusion_matrix.BinaryConfusionMatrix)

        df = cm.to_dataframe()
        df_norm = cm.to_dataframe(normalized=True)
        assert(df_norm.sum(axis=1, skipna=False).fillna(1).sum() == len(df))

    def test_pandas_confusion_matrix_auto_labeled(self):
        y_true = ['rabbit', 'cat', 'rabbit', 'rabbit', 'cat', 'dog', 'dog', 'rabbit', 'rabbit', 'cat', 'dog', 'rabbit']
        y_pred = ['cat', 'cat', 'rabbit', 'dog', 'cat', 'rabbit', 'dog', 'cat', 'rabbit', 'cat', 'rabbit', 'rabbit']

        cm = ConfusionMatrix(y_true, y_pred)
        assert(isinstance(cm, pdml.confusion_matrix.LabeledConfusionMatrix))

    def test_pandas_confusion_matrix_auto_binary(self):
        y_true = [True, True, False, False, False, True, False, True, True,
                  False, True, False, False, False, False, False, True, False,
                  True, True, True, True, False, False, False, True, False,
                  True, False, False, False, False, True, True, False, False,
                  False, True, True, True, True, False, False, False, False,
                  True, False, False, False, False, False, False, False, False,
                  False, True, True, False, True, False, True, True, True,
                  False, False, True, False, True, False, False, True, False,
                  False, False, False, False, False, False, False, True, False,
                  True, True, True, True, False, False, True, False, True,
                  True, False, True, False, True, False, False, True, True,
                  False, False, True, True, False, False, False, False, False,
                  False, True, True, False]

        y_pred = [False, False, False, False, False, True, False, False, True,
                  False, True, False, False, False, False, False, False, False,
                  True, True, True, True, False, False, False, False, False,
                  False, False, False, False, False, True, False, False, False,
                  False, True, False, False, False, False, False, False, False,
                  True, False, False, False, False, False, False, False, False,
                  False, True, False, False, False, False, False, False, False,
                  False, False, True, False, False, False, False, True, False,
                  False, False, False, False, False, False, False, True, False,
                  False, True, False, False, False, False, True, False, True,
                  True, False, False, False, True, False, False, True, True,
                  False, False, True, True, False, False, False, False, False,
                  False, True, False, False]

        cm = ConfusionMatrix(y_true, y_pred)
        assert(isinstance(cm, pdml.confusion_matrix.BinaryConfusionMatrix))

    def test_plot(self):

        try:
            import matplotlib.pyplot        # noqa
        except ImportError:
            import nose
            raise nose.SkipTest()

        y_true = ['rabbit', 'cat', 'rabbit', 'rabbit', 'cat', 'dog',
                  'dog', 'rabbit', 'rabbit', 'cat', 'dog', 'rabbit']
        y_pred = ['cat', 'cat', 'rabbit', 'dog', 'cat', 'rabbit', 'dog',
                  'cat', 'rabbit', 'cat', 'rabbit', 'rabbit']

        cm = ConfusionMatrix(y_true, y_pred)

        # check plot works
        cm.plot()
        cm.plot(backend='seaborn')

        with pytest.raises(ValueError):
            cm.plot(backend='xxx')



#!/usr/bin/env python

from distutils.version import LooseVersion

import pandas as pd
from pandas.api.types import is_list_like, is_integer_dtype         # noqa

PANDAS_VERSION = LooseVersion(pd.__version__)

if PANDAS_VERSION >= LooseVersion('0.23'):
    _PANDAS_ge_023 = True
else:
    _PANDAS_ge_023 = False

if PANDAS_VERSION >= LooseVersion('0.22'):
    _PANDAS_ge_022 = True
else:
    _PANDAS_ge_022 = False

if PANDAS_VERSION >= LooseVersion('0.21'):
    from pandas.util import Appender, cache_readonly              # noqa
    import pandas.plotting as plotting                            # noqa
    _PANDAS_ge_021 = True
else:
    from pandas.util.decorators import Appender, cache_readonly   # noqa
    import pandas.tools.plotting as plotting                      # noqa
    _PANDAS_ge_021 = False

try:
    import sklearn
    _SKLEARN_INSTALLED = True
    _SKLEARN_ge_019 = sklearn.__version__ >= LooseVersion('0.19')

except ImportError:
    _SKLEARN_INSTALLED = False
    _SKLEARN_ge_019 = False


try:
    import imblearn                 # noqa
    _IMBLEARN_INSTALLED = True
except ImportError:
    _IMBLEARN_INSTALLED = False



#!/usr/bin/python
# -*- coding: utf8 -*-

import numpy as np
from pandas.compat import range


def binom_interval(success, total, confint=0.95):
    """
    Compute two-sided binomial confidence interval in Python. Based on R's binom.test.
    """

    from scipy.stats import beta

    quantile = (1 - confint) / 2.
    lower = beta.ppf(quantile, success, total - success + 1)
    upper = beta.ppf(1 - quantile, success + 1, total - success)
    return (lower, upper)


def choose(n, k):
    """
    A fast way to calculate binomial coefficients by Andrew Dalke (contrib).
    """
    if np.isnan(n):
        return np.nan
    else:
        n = np.int64(n)

    if 0 <= k <= n:
        ntok = 1
        ktok = 1
        for t in range(1, min(k, n - k) + 1):
            ntok *= n
            ktok *= t
            n -= 1
        return ntok // ktok
    else:
        return 0


def class_agreement(df):
    """
    Inspired from R package e1071 matchClassed.R classAgreement
    """
    n = df.sum().sum()
    nj = df.sum(axis=0)
    ni = df.sum(axis=1)
    m = min(len(ni), len(nj))

    p0 = np.float64(np.diag(df.iloc[0:m, 0:m]).sum()) / n
    pc = np.float64((ni.iloc[0:m] * nj.iloc[0:m]).sum()) / (n**2)

    n2 = choose(n, 2)

    rand = np.float64(1) + ((df**2).sum().sum() - ((ni**2).sum() + (nj**2).sum()) / 2) / n2
    nis2 = ni[ni > 1].map(lambda x: choose(int(x), 2)).sum()
    njs2 = nj[nj > 1].map(lambda x: choose(int(x), 2)).sum()

    num = df[df > 1].dropna(axis=[0, 1], thresh=1).applymap(lambda n: choose(n, 2)).sum().sum() - np.float64(nis2 * njs2) / n2
    den = (np.float64(nis2 + njs2) / 2 - np.float64(nis2 * njs2) / n2)
    crand = num / den

    return({
        "diag": p0,
        "kappa": np.float64(p0 - pc) / (1 - pc),
        "rand": rand,
        "crand": crand
    })


def prop_test(df):
    """
    Inspired from R package caret confusionMatrix.R
    """
    from scipy.stats import binom

    x = np.diag(df).sum()
    n = df.sum().sum()
    p = (df.sum(axis=0) / df.sum().sum()).max()
    d = {
        "statistic": x,  # number of successes
        "parameter": n,  # number of trials
        "null.value": p,  # probability of success
        "p.value": binom.sf(x - 1, n, p),  # see https://en.wikipedia.org/wiki/Binomial_test
    }
    return(d)



#!/usr/bin/env python

from pandas_ml.core.accessor import _AccessorMethods


class IsotonicMethods(_AccessorMethods):
    """
    Accessor to ``sklearn.isotonic``.
    """

    _module_name = 'sklearn.isotonic'

    @property
    def IsotonicRegression(self):
        """``sklearn.isotonic.IsotonicRegression``"""
        return self._module.IsotonicRegression

    def isotonic_regression(self, *args, **kwargs):
        """
        Call ``sklearn.isotonic.isotonic_regression`` using automatic mapping.

        - ``y``: ``ModelFrame.target``
        """
        func = self._module.isotonic_regression
        target = self._target
        _y = func(target.values, *args, **kwargs)
        _y = self._constructor_sliced(_y, index=target.index)
        return _y

    def check_increasing(self, *args, **kwargs):
        """
        Call ``sklearn.isotonic.check_increasing`` using automatic mapping.

        - ``x``: ``ModelFrame.index``
        - ``y``: ``ModelFrame.target``
        """
        func = self._module.check_increasing
        target = self._target
        return func(target.index, target.values, *args, **kwargs)



#!/usr/bin/env python

try:
    import sklearn.base as base

    _BaseEstimator = base.BaseEstimator
    _ClassifierMixin = base.ClassifierMixin
    _ClusterMixin = base.ClusterMixin
    _RegressorMixin = base.RegressorMixin
    _TransformerMixin = base.TransformerMixin

except ImportError:
    # for ReadTheDoc, unable to use mock because of metaclass

    class _BaseEstimator(object):
        pass

    class _ClassifierMixin(object):
        pass

    class _ClusterMixin(object):
        pass

    class _RegressorMixin(object):
        pass

    class _TransformerMixin(object):
        pass



#!/usr/bin/env python

from pandas_ml.core import ModelFrame, ModelSeries       # noqa
from pandas_ml.tools import info                         # noqa
from pandas_ml.version import version as __version__     # noqa
import pandas_ml.compat as compat                        # noqa

from pandas_ml.confusion_matrix.cm import ConfusionMatrix  # noqa



#!/usr/bin/env python

from pandas_ml.core.accessor import _AccessorMethods


class PipelineMethods(_AccessorMethods):
    """
    Accessor to ``sklearn.pipeline``.
    """

    _module_name = 'sklearn.pipeline'

    @property
    def make_pipeline(self):
        """``sklearn.pipeline.make_pipeline``"""
        # not included in __all__
        return self._module.make_pipeline

    @property
    def make_union(self):
        """``sklearn.pipeline.make_union``"""
        # not included in __all__
        return self._module.make_union



#!/usr/bin/env python

import numpy as np

from pandas_ml.core.accessor import _AccessorMethods, _attach_methods
from pandas_ml.compat import (_SKLEARN_INSTALLED, is_integer_dtype)


if _SKLEARN_INSTALLED:
    import sklearn.preprocessing as pp
    _keep_col_classes = [pp.Binarizer,
                         pp.FunctionTransformer,
                         pp.Imputer,
                         pp.KernelCenterer,
                         pp.LabelEncoder,
                         pp.MaxAbsScaler,
                         pp.MinMaxScaler,
                         pp.Normalizer,
                         pp.RobustScaler,
                         pp.StandardScaler]
else:
    _keep_col_classes = []


class PreprocessingMethods(_AccessorMethods):
    """
    Accessor to ``sklearn.preprocessing``.
    """

    _module_name = 'sklearn.preprocessing'

    def _keep_existing_columns(self, estimator):
        """
        Check whether estimator should preserve existing column names
        """
        return estimator.__class__ in _keep_col_classes

    def add_dummy_feature(self, value=1.0):
        """
        Call ``sklearn.preprocessing.add_dummy_feature`` using automatic mapping.

        - ``X``: ``ModelFrame.data``
        """
        from pandas_ml.core.series import ModelSeries
        from pandas_ml.core.frame import ModelFrame

        func = self._module.add_dummy_feature

        if isinstance(self._df, ModelSeries):
            data = self._df.to_frame()
            constructor = ModelFrame
        else:
            data = self._data
            constructor = self._constructor

        result = func(data.values, value=value)
        result = constructor(result, index=data.index)
        columns = result.columns[:-len(data.columns)].append(data.columns)
        result.columns = columns
        return result


_preprocessing_methods = ['binarize', 'normalize', 'scale']


def _wrap_func(func, func_name):
    def f(self, *args, **kwargs):
        from pandas_ml.core.frame import ModelFrame
        if isinstance(self._df, ModelFrame):
            values = self._data.values

            if is_integer_dtype(values):
                # integer raises an error in normalize
                values = values.astype(np.float)

            result = func(values, *args, **kwargs)
            result = self._constructor(result, index=self._data.index,
                                       columns=self._data.columns)
        else:
            # ModelSeries
            values = np.atleast_2d(self._df.values)
            if is_integer_dtype(values):
                values = values.astype(np.float)

            result = func(values, *args, **kwargs)
            result = self._constructor(result[0], index=self._df.index,
                                       name=self._df.name)
        return result
    f.__doc__ = (
        """
        Call ``%s`` using automatic mapping.

        - ``X``: ``ModelFrame.data``
        """ % func_name)
    return f


_attach_methods(PreprocessingMethods, _wrap_func, _preprocessing_methods)



#!/usr/bin/env python
import pytest

import sklearn.datasets as datasets
import sklearn.linear_model as lm
import sklearn.preprocessing as pp

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestLinearModel(tm.TestCase):

    def test_objectmapper(self):
        df = pdml.ModelFrame([])
        self.assertIs(df.linear_model.ARDRegression, lm.ARDRegression)
        self.assertIs(df.linear_model.BayesianRidge, lm.BayesianRidge)
        self.assertIs(df.linear_model.ElasticNet, lm.ElasticNet)
        self.assertIs(df.linear_model.ElasticNetCV, lm.ElasticNetCV)

        self.assertIs(df.linear_model.HuberRegressor, lm.HuberRegressor)

        self.assertIs(df.linear_model.Lars, lm.Lars)
        self.assertIs(df.linear_model.LarsCV, lm.LarsCV)
        self.assertIs(df.linear_model.Lasso, lm.Lasso)
        self.assertIs(df.linear_model.LassoCV, lm.LassoCV)
        self.assertIs(df.linear_model.LassoLars, lm.LassoLars)
        self.assertIs(df.linear_model.LassoLarsCV, lm.LassoLarsCV)
        self.assertIs(df.linear_model.LassoLarsIC, lm.LassoLarsIC)

        self.assertIs(df.linear_model.LinearRegression, lm.LinearRegression)
        self.assertIs(df.linear_model.LogisticRegression, lm.LogisticRegression)
        self.assertIs(df.linear_model.LogisticRegressionCV, lm.LogisticRegressionCV)
        self.assertIs(df.linear_model.MultiTaskLasso, lm.MultiTaskLasso)
        self.assertIs(df.linear_model.MultiTaskElasticNet, lm.MultiTaskElasticNet)
        self.assertIs(df.linear_model.MultiTaskLassoCV, lm.MultiTaskLassoCV)
        self.assertIs(df.linear_model.MultiTaskElasticNetCV, lm.MultiTaskElasticNetCV)

        self.assertIs(df.linear_model.OrthogonalMatchingPursuit, lm.OrthogonalMatchingPursuit)
        self.assertIs(df.linear_model.OrthogonalMatchingPursuitCV, lm.OrthogonalMatchingPursuitCV)
        self.assertIs(df.linear_model.PassiveAggressiveClassifier, lm.PassiveAggressiveClassifier)
        self.assertIs(df.linear_model.PassiveAggressiveRegressor, lm.PassiveAggressiveRegressor)

        self.assertIs(df.linear_model.Perceptron, lm.Perceptron)
        self.assertIs(df.linear_model.RandomizedLasso, lm.RandomizedLasso)
        self.assertIs(df.linear_model.RandomizedLogisticRegression, lm.RandomizedLogisticRegression)
        self.assertIs(df.linear_model.RANSACRegressor, lm.RANSACRegressor)
        self.assertIs(df.linear_model.Ridge, lm.Ridge)
        self.assertIs(df.linear_model.RidgeClassifier, lm.RidgeClassifier)
        self.assertIs(df.linear_model.RidgeClassifierCV, lm.RidgeClassifierCV)
        self.assertIs(df.linear_model.RidgeCV, lm.RidgeCV)
        self.assertIs(df.linear_model.SGDClassifier, lm.SGDClassifier)
        self.assertIs(df.linear_model.SGDRegressor, lm.SGDRegressor)
        self.assertIs(df.linear_model.TheilSenRegressor, lm.TheilSenRegressor)

    def test_lars_path(self):
        diabetes = datasets.load_diabetes()
        df = pdml.ModelFrame(diabetes)

        result = df.linear_model.lars_path()
        expected = lm.lars_path(diabetes.data, diabetes.target)

        self.assertEqual(len(result), 3)
        tm.assert_numpy_array_equal(result[0], expected[0])
        self.assertEqual(result[1], expected[1])
        self.assertIsInstance(result[1], list)
        self.assertIsInstance(result[2], pdml.ModelFrame)
        tm.assert_index_equal(result[2].index, df.data.columns)
        tm.assert_numpy_array_equal(result[2].values, expected[2])

    def test_lasso_path(self):
        diabetes = datasets.load_diabetes()
        df = pdml.ModelFrame(diabetes)

        result = df.linear_model.lasso_path()
        expected = lm.lasso_path(diabetes.data, diabetes.target)

        self.assertEqual(len(result), 3)
        tm.assert_numpy_array_equal(result[0], expected[0])
        self.assertIsInstance(result[1], pdml.ModelFrame)
        tm.assert_index_equal(result[1].index, df.data.columns)
        self.assert_numpy_array_almost_equal(result[1].values, expected[1])
        self.assert_numpy_array_almost_equal(result[2], expected[2])

        result = df.linear_model.lasso_path(return_models=True)
        expected = lm.lasso_path(diabetes.data, diabetes.target, return_models=True)
        self.assertEqual(len(result), len(expected))
        self.assertIsInstance(result, tuple)
        tm.assert_numpy_array_equal(result[0], result[0])
        tm.assert_numpy_array_equal(result[1], result[1])
        tm.assert_numpy_array_equal(result[2], result[2])

    def test_lasso_stability_path(self):
        diabetes = datasets.load_diabetes()
        df = pdml.ModelFrame(diabetes)

        result = df.linear_model.lasso_stability_path(random_state=self.random_state)
        expected = lm.lasso_stability_path(diabetes.data, diabetes.target,
                                           random_state=self.random_state)

        self.assertEqual(len(result), 2)
        tm.assert_numpy_array_equal(result[0], expected[0])

        self.assertIsInstance(result[1], pdml.ModelFrame)
        tm.assert_index_equal(result[1].index, df.data.columns)
        tm.assert_numpy_array_equal(result[1].values, expected[1])

    def test_orthogonal_mp(self):
        diabetes = datasets.load_diabetes()
        df = pdml.ModelFrame(diabetes)

        result = df.linear_model.orthogonal_mp()
        expected = lm.orthogonal_mp(diabetes.data, diabetes.target)
        tm.assert_numpy_array_equal(result, expected)

        result = df.linear_model.orthogonal_mp(return_path=True)
        expected = lm.orthogonal_mp(diabetes.data, diabetes.target, return_path=True)
        tm.assert_numpy_array_equal(result, expected)

    def test_orthogonal_mp_gram(self):
        diabetes = datasets.load_diabetes()
        df = pdml.ModelFrame(diabetes)

        result = df.linear_model.orthogonal_mp()

        gram = diabetes.data.T.dot(diabetes.data)
        Xy = diabetes.data.T.dot(diabetes.target)
        expected = lm.orthogonal_mp_gram(gram, Xy)
        self.assert_numpy_array_almost_equal(result, expected)

    @pytest.mark.parametrize("algo", ['LinearRegression', 'Ridge',
                             'RidgeCV', 'Lasso'])
    def test_Regresions(self, algo):
        diabetes = datasets.load_diabetes()
        df = pdml.ModelFrame(diabetes)

        mod1 = getattr(df.linear_model, algo)()
        mod2 = getattr(lm, algo)()

        df.fit(mod1)
        mod2.fit(diabetes.data, diabetes.target)

        result = df.predict(mod1)
        expected = mod2.predict(diabetes.data)

        self.assertIsInstance(result, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(result.values, expected)

    def test_Lasso_Path(self):
        diabetes = datasets.load_diabetes()
        X = diabetes.data
        y = diabetes.target
        X /= X.std(axis=0)

        df = pdml.ModelFrame(diabetes)
        df.data /= df.data.std(axis=0, ddof=False)

        self.assert_numpy_array_almost_equal(df.data.values, X)

        eps = 5e-3
        expected = lm.lasso_path(X, y, eps, fit_intercept=False)
        result = df.lm.lasso_path(eps=eps, fit_intercept=False)
        self.assert_numpy_array_almost_equal(expected[0], result[0])
        self.assert_numpy_array_almost_equal(expected[1], result[1])
        self.assert_numpy_array_almost_equal(expected[2], result[2])

        expected = lm.enet_path(X, y, eps=eps, l1_ratio=0.8, fit_intercept=False)
        result = df.lm.enet_path(eps=eps, l1_ratio=0.8, fit_intercept=False)
        self.assert_numpy_array_almost_equal(expected[0], result[0])
        self.assert_numpy_array_almost_equal(expected[1], result[1])
        self.assert_numpy_array_almost_equal(expected[2], result[2])

        expected = lm.enet_path(X, y, eps=eps, l1_ratio=0.8, positive=True, fit_intercept=False)
        result = df.lm.enet_path(eps=eps, l1_ratio=0.8, positive=True, fit_intercept=False)
        self.assert_numpy_array_almost_equal(expected[0], result[0])
        self.assert_numpy_array_almost_equal(expected[1], result[1])
        self.assert_numpy_array_almost_equal(expected[2], result[2])

        expected = lm.lars_path(X, y, method='lasso', verbose=True)
        result = df.lm.lars_path(method='lasso', verbose=True)
        self.assert_numpy_array_almost_equal(expected[0], result[0])
        self.assert_numpy_array_almost_equal(expected[1], result[1])
        self.assert_numpy_array_almost_equal(expected[2], result[2])

    @pytest.mark.parametrize("criterion", ['aic', 'bic'])
    def test_LassoCV(self, criterion):
        diabetes = datasets.load_diabetes()
        X = diabetes.data
        y = diabetes.target

        X = pp.normalize(X)

        df = pdml.ModelFrame(diabetes)
        df.data = df.data.pp.normalize()

        mod1 = lm.LassoLarsIC(criterion=criterion)
        mod1.fit(X, y)

        mod2 = df.lm.LassoLarsIC(criterion=criterion)
        df.fit(mod2)
        self.assertAlmostEqual(mod1.alpha_, mod2.alpha_)

        expected = mod1.predict(X)
        predicted = df.predict(mod2)
        self.assertIsInstance(predicted, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(predicted.values, expected)

    def test_SGD(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        clf1 = lm.SGDClassifier(alpha=0.001, n_iter=100, random_state=self.random_state)
        clf1.fit(iris.data, iris.target)
        clf2 = df.lm.SGDClassifier(alpha=0.001, n_iter=100, random_state=self.random_state)
        df.fit(clf2)

        expected = clf1.predict(iris.data)
        predicted = df.predict(clf2)
        self.assertIsInstance(predicted, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(predicted.values, expected)

    def test_Perceptron(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        clf1 = lm.Perceptron(alpha=0.001, n_iter=100).fit(iris.data, iris.target)
        clf2 = df.lm.Perceptron(alpha=0.001, n_iter=100)
        df.fit(clf2)

        expected = clf1.predict(iris.data)
        predicted = df.predict(clf2)
        self.assertIsInstance(predicted, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(predicted.values, expected)



#!/usr/bin/env python
import pytest
import numpy as np
import pandas as pd

import sklearn.datasets as datasets
import sklearn.preprocessing as pp

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestPreprocessing(tm.TestCase):

    def test_objectmapper(self):
        df = pdml.ModelFrame([])
        self.assertIs(df.preprocessing.Binarizer, pp.Binarizer)
        self.assertIs(df.preprocessing.FunctionTransformer,
                      pp.FunctionTransformer)
        self.assertIs(df.preprocessing.Imputer, pp.Imputer)
        self.assertIs(df.preprocessing.KernelCenterer, pp.KernelCenterer)
        self.assertIs(df.preprocessing.LabelBinarizer, pp.LabelBinarizer)
        self.assertIs(df.preprocessing.LabelEncoder, pp.LabelEncoder)
        self.assertIs(df.preprocessing.MultiLabelBinarizer, pp.MultiLabelBinarizer)
        self.assertIs(df.preprocessing.MaxAbsScaler, pp.MaxAbsScaler)
        self.assertIs(df.preprocessing.MinMaxScaler, pp.MinMaxScaler)
        self.assertIs(df.preprocessing.Normalizer, pp.Normalizer)
        self.assertIs(df.preprocessing.OneHotEncoder, pp.OneHotEncoder)
        self.assertIs(df.preprocessing.PolynomialFeatures, pp.PolynomialFeatures)
        self.assertIs(df.preprocessing.RobustScaler, pp.RobustScaler)
        self.assertIs(df.preprocessing.StandardScaler, pp.StandardScaler)

    def test_add_dummy_feature(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        result = df.preprocessing.add_dummy_feature()
        expected = pp.add_dummy_feature(iris.data)

        self.assertIsInstance(result, pdml.ModelFrame)
        self.assert_numpy_array_almost_equal(result.data.values, expected)

        result = df.preprocessing.add_dummy_feature(value=2)
        expected = pp.add_dummy_feature(iris.data, value=2)

        self.assertIsInstance(result, pdml.ModelFrame)
        self.assert_numpy_array_almost_equal(result.data.values, expected)
        tm.assert_index_equal(result.columns[1:], df.data.columns)

        s = df['sepal length (cm)']
        self.assertIsInstance(s, pdml.ModelSeries)
        result = s.preprocessing.add_dummy_feature()
        expected = pp.add_dummy_feature(iris.data[:, [0]])

        self.assertIsInstance(result, pdml.ModelFrame)
        self.assert_numpy_array_almost_equal(result.values, expected)
        self.assertEqual(result.columns[1], 'sepal length (cm)')

    def test_binarize(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        result = df.preprocessing.binarize()
        expected = pp.binarize(iris.data)

        self.assertIsInstance(result, pdml.ModelFrame)
        self.assert_numpy_array_almost_equal(result.data.values, expected)
        tm.assert_index_equal(result.columns, df.data.columns)

        result = df.preprocessing.binarize(threshold=5)
        expected = pp.binarize(iris.data, threshold=5)

        self.assertIsInstance(result, pdml.ModelFrame)
        self.assert_numpy_array_almost_equal(result.data.values, expected)
        tm.assert_index_equal(result.columns, df.data.columns)

        s = df['sepal length (cm)']
        self.assertIsInstance(s, pdml.ModelSeries)
        result = s.preprocessing.binarize()
        expected = pp.binarize(iris.data[:, 0].reshape(-1, 1))

        self.assertIsInstance(result, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(result.values, expected.flatten())
        self.assertEqual(result.name, 'sepal length (cm)')

        result = s.preprocessing.binarize(threshold=6)
        expected = pp.binarize(iris.data[:, 0].reshape(-1, 1), threshold=6)

        self.assertIsInstance(result, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(result.values, expected.flatten())
        self.assertEqual(result.name, 'sepal length (cm)')

    def test_normalize(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        result = df.preprocessing.normalize()
        expected = pp.normalize(iris.data)

        self.assertIsInstance(result, pdml.ModelFrame)
        self.assert_numpy_array_almost_equal(result.data.values, expected)
        tm.assert_index_equal(result.columns, df.data.columns)

        s = df['sepal length (cm)']
        self.assertIsInstance(s, pdml.ModelSeries)
        result = s.preprocessing.normalize()
        expected = pp.normalize(np.atleast_2d(iris.data[:, 0]))[0]

        self.assertIsInstance(result, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(result.values, expected)
        self.assertEqual(result.name, 'sepal length (cm)')

    def test_normalize_abbr(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        result = df.pp.normalize()
        expected = pp.normalize(iris.data)

        self.assertIsInstance(result, pdml.ModelFrame)
        self.assert_numpy_array_almost_equal(result.data.values, expected)
        tm.assert_index_equal(result.columns, df.data.columns)

        s = df['sepal length (cm)']
        self.assertIsInstance(s, pdml.ModelSeries)
        result = s.pp.normalize()
        expected = pp.normalize(np.atleast_2d(iris.data[:, 0]))[0]

        self.assertIsInstance(result, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(result.values, expected)
        self.assertEqual(result.name, 'sepal length (cm)')

    def test_scale(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        result = df.preprocessing.scale()
        expected = pp.scale(iris.data)

        self.assertIsInstance(result, pdml.ModelFrame)
        self.assert_numpy_array_almost_equal(result.data.values, expected)
        tm.assert_index_equal(result.columns, df.data.columns)

        s = df['sepal length (cm)']
        self.assertIsInstance(s, pdml.ModelSeries)
        result = s.preprocessing.scale()
        expected = pp.scale(np.atleast_2d(iris.data[:, 0]))[0]

        self.assertIsInstance(result, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(result.values, expected)
        self.assertEqual(result.name, 'sepal length (cm)')

    def test_preprocessing_assignment(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        original_columns = df.data.columns
        df['sepal length (cm)'] = df['sepal length (cm)'].preprocessing.binarize(threshold=6)
        self.assertIsInstance(df, pdml.ModelFrame)
        binarized = pp.binarize(np.atleast_2d(iris.data[:, 0]), threshold=6)
        expected = np.hstack([binarized.T, iris.data[:, 1:]])
        self.assert_numpy_array_almost_equal(df.data.values, expected)
        tm.assert_index_equal(df.data.columns, original_columns)

        # recreate data
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        target_columns = ['sepal length (cm)', 'sepal width (cm)']
        df[target_columns] = df[target_columns].preprocessing.binarize(threshold=6)
        self.assertIsInstance(df, pdml.ModelFrame)
        binarized = pp.binarize(iris.data[:, 0:2], threshold=6)
        expected = np.hstack([binarized, iris.data[:, 2:]])
        self.assert_numpy_array_almost_equal(df.data.values, expected)
        tm.assert_index_equal(df.data.columns, original_columns)

    def _assert_transform(self, df, exp_data, model1, model2,
                          check_target=True):
        df.fit(model1)
        model2.fit(exp_data)

        result = df.transform(model1)
        expected = model2.transform(exp_data)

        self.assertIsInstance(result, pdml.ModelFrame)

        if df.has_target():
            # target is unchanged
            tm.assert_series_equal(df.target, result.target)
        else:
            self.assertIsNone(result.target)

        self.assert_numpy_array_almost_equal(result.data.values, expected)
        # index and columns are kept
        tm.assert_index_equal(result.index, df.index)
        tm.assert_index_equal(result.columns, df.columns)

    def _assert_fit_transform(self, df, exp_data, model1, model2):
        result = df.fit_transform(model1)
        expected = model2.fit_transform(exp_data)

        self.assertIsInstance(result, pdml.ModelFrame)
        # target is unchanged
        if df.has_target():
            # target is unchanged
            tm.assert_series_equal(df.target, result.target)
        else:
            self.assertIsNone(result.target)

        self.assert_numpy_array_almost_equal(result.data.values, expected)
        # index and columns are kept
        tm.assert_index_equal(result.index, df.index)
        tm.assert_index_equal(result.columns, df.columns)

    @pytest.mark.parametrize("algo", ['Binarizer', 'Imputer', 'KernelCenterer',
                                      'MaxAbsScaler', 'MinMaxScaler',
                                      'Normalizer', 'RobustScaler',
                                      'StandardScaler'])
    def test_transform(self, algo):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        mod1 = getattr(df.preprocessing, algo)()
        mod2 = getattr(pp, algo)()

        self._assert_transform(df, iris.data, mod1, mod2)

        mod1 = getattr(df.preprocessing, algo)()
        mod2 = getattr(pp, algo)()
        self._assert_fit_transform(df, iris.data, mod1, mod2)

    def test_transform_1d_frame_int(self):
        arr = np.array([1, 2, 3, 1, 2, 3, 1, 2, 3])
        idx = pd.Index('a b c d e f g h i'.split(' '))
        df = pdml.ModelFrame(arr, index=idx, columns=['X'])
        self.assertEqual(len(df.columns), 1)

        # reshape arr to 2d
        arr = arr.reshape(-1, 1)

        if pd.compat.PY3:
            models = ['Binarizer', 'Imputer', 'StandardScaler']
            # MinMaxScalar raises TypeError in ufunc
        else:
            models = ['Binarizer', 'Imputer', 'StandardScaler', 'MinMaxScaler']

        for model in models:
            mod1 = getattr(df.preprocessing, model)()
            mod2 = getattr(pp, model)()

            self._assert_transform(df, arr, mod1, mod2)

            mod1 = getattr(df.preprocessing, model)()
            mod2 = getattr(pp, model)()
            self._assert_fit_transform(df, arr, mod1, mod2)

    @pytest.mark.parametrize("algo", ['Binarizer', 'Imputer', 'StandardScaler',
                                      'MinMaxScaler'])
    def test_transform_1d_frame_float(self, algo):
        arr = np.array([1, 2, 3, 1, 2, 3, 1, 2, 3], dtype=np.float)
        idx = pd.Index('a b c d e f g h i'.split(' '))
        df = pdml.ModelFrame(arr, index=idx, columns=['X'])
        self.assertEqual(len(df.columns), 1)

        # reshape arr to 2d
        arr = arr.reshape(-1, 1)

        mod1 = getattr(df.preprocessing, algo)()
        mod2 = getattr(pp, algo)()
        self._assert_transform(df, arr, mod1, mod2)

        mod1 = getattr(df.preprocessing, algo)()
        mod2 = getattr(pp, algo)()
        self._assert_fit_transform(df, arr, mod1, mod2)

    def test_transform_series_int(self):
        arr = np.array([1, 2, 3, 1, 2, 3, 1, 2, 3])
        s = pdml.ModelSeries(arr, index='a b c d e f g h i'.split(' '))

        # reshape arr to 2d
        arr = arr.reshape(-1, 1)

        if pd.compat.PY3:
            models = ['Binarizer', 'Imputer', 'StandardScaler']
            # MinMaxScalar raises TypeError in ufunc
        else:
            models = ['Binarizer', 'Imputer', 'StandardScaler', 'MinMaxScaler']

        for model in models:
            mod1 = getattr(s.preprocessing, model)()
            mod2 = getattr(pp, model)()
            s.fit(mod1)
            mod2.fit(arr)

            result = s.transform(mod1)
            expected = mod2.transform(arr).flatten()

            self.assertIsInstance(result, pdml.ModelSeries)
            self.assert_numpy_array_almost_equal(result.values, expected)

            mod1 = getattr(s.preprocessing, model)()
            mod2 = getattr(pp, model)()

            result = s.fit_transform(mod1)
            expected = mod2.fit_transform(arr).flatten()

            self.assertIsInstance(result, pdml.ModelSeries)
            self.assert_numpy_array_almost_equal(result.values, expected)

    @pytest.mark.parametrize("algo", ['Binarizer', 'Imputer', 'StandardScaler',
                                      'MinMaxScaler'])
    def test_transform_series_float(self, algo):
        arr = np.array([1, 2, 3, 1, 2, 3, 1, 2, 3], dtype=np.float)
        s = pdml.ModelSeries(arr, index='a b c d e f g h i'.split(' '))

        # reshape arr to 2d
        arr = arr.reshape(-1, 1)

        mod1 = getattr(s.preprocessing, algo)()
        mod2 = getattr(pp, algo)()

        s.fit(mod1)
        mod2.fit(arr)

        result = s.transform(mod1)
        expected = mod2.transform(arr).flatten()

        self.assertIsInstance(result, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(result.values, expected)

        mod1 = getattr(s.preprocessing, algo)()
        mod2 = getattr(pp, algo)()

        result = s.fit_transform(mod1)
        expected = mod2.fit_transform(arr).flatten()

        self.assertIsInstance(result, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(result.values, expected)

    def test_FunctionTransformer(self):
        iris = datasets.load_iris()
        df = pdml.ModelFrame(iris)

        mod1 = df.pp.FunctionTransformer(func=lambda x: x + 1)
        df.fit(mod1)
        result = df.transform(mod1)

        exp = df.copy()
        exp.data = exp.data + 1

        self.assertIsInstance(result, pdml.ModelFrame)
        tm.assert_frame_equal(result, exp)

    def test_Imputer(self):
        arr = np.array([1, np.nan, 3, 2])
        s = pdml.ModelSeries(arr)

        mod1 = s.pp.Imputer(axis=0)
        s.fit(mod1)
        result = s.transform(mod1)

        expected = np.array([1, 2, 3, 2])

        self.assertIsInstance(result, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(result.values, expected)

        mod1 = s.pp.Imputer(axis=0)
        result = s.fit_transform(mod1)

        self.assertIsInstance(result, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(result.values, expected)

    def test_LabelBinarizer(self):
        arr = np.array([1, 2, 3, 2])
        s = pdml.ModelSeries(arr, index=['a', 'b', 'c', 'd'])

        mod1 = s.pp.LabelBinarizer()
        s.fit(mod1)
        result = s.transform(mod1)

        expected = np.array([[1, 0, 0], [0, 1, 0], [0, 0, 1], [0, 1, 0]])

        self.assertIsInstance(result, pdml.ModelFrame)
        self.assert_numpy_array_almost_equal(result.values, expected)
        tm.assert_index_equal(result.index, s.index)

        mod1 = s.pp.LabelBinarizer()
        result = s.fit_transform(mod1)

        self.assertIsInstance(result, pdml.ModelFrame)
        self.assert_numpy_array_almost_equal(result.values, expected)

        inversed = result.inverse_transform(mod1)
        self.assertIsInstance(inversed, pdml.ModelFrame)
        self.assert_numpy_array_almost_equal(inversed.values.flatten(), arr)
        tm.assert_index_equal(result.index, s.index)

    def test_LabelBinarizer2(self):
        arr = np.array(['X', 'Y', 'Z', 'X'])
        s = pdml.ModelSeries(arr)

        lb = s.preprocessing.LabelBinarizer()
        s.fit(lb)

        binarized = s.transform(lb)
        self.assertIsInstance(binarized, pdml.ModelFrame)

        expected = pd.DataFrame({0: [1, 0, 0, 1], 1: [0, 1, 0, 0], 2: [0, 0, 1, 0]})
        tm.assert_frame_equal(binarized, expected)

        df = pdml.ModelFrame(datasets.load_iris())
        df.target.fit(lb)
        binarized = df.target.transform(lb)

        expected = pd.DataFrame({0: [1] * 50 + [0] * 100,
                                 1: [0] * 50 + [1] * 50 + [0] * 50,
                                 2: [0] * 100 + [1] * 50})
        tm.assert_frame_equal(binarized, expected)

        df = pdml.ModelFrame(datasets.load_iris())
        df.target.fit(lb)
        df.target = df.target.transform(lb)
        self.assertEqual(df.shape, (150, 7))
        tm.assert_frame_equal(df.target, expected)

    def test_LabelEncoder_frame(self):
        arr = np.array(['X', 'Y', 'Z', 'X'])
        df = pdml.ModelFrame(arr, index=['a', 'b', 'c', 'd'], columns=['A'])

        mod1 = df.pp.LabelEncoder()
        df.fit(mod1)
        result = df.transform(mod1)

        expected = np.array([0, 1, 2, 0]).reshape(-1, 1)

        self.assertIsInstance(result, pdml.ModelFrame)
        self.assert_numpy_array_almost_equal(result.values, expected)
        tm.assert_index_equal(result.columns, df.columns)
        tm.assert_index_equal(result.index, df.index)

        mod1 = df.pp.LabelEncoder()
        result = df.fit_transform(mod1)

        self.assertIsInstance(result, pdml.ModelFrame)
        self.assert_numpy_array_almost_equal(result.values, expected)
        tm.assert_index_equal(result.columns, df.columns)
        tm.assert_index_equal(result.index, df.index)

        inversed = result.inverse_transform(mod1)
        self.assertIsInstance(inversed, pdml.ModelFrame)
        tm.assert_frame_equal(inversed, df)

    def test_LabelEncoder_series(self):
        arr = np.array(['X', 'Y', 'Z', 'X'])
        s = pdml.ModelSeries(arr, index=['a', 'b', 'c', 'd'])

        mod1 = s.pp.LabelEncoder()
        s.fit(mod1)
        result = s.transform(mod1)

        expected = np.array([0, 1, 2, 0])

        self.assertIsInstance(result, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(result.values, expected)
        tm.assert_index_equal(result.index, s.index)

        mod1 = s.pp.LabelEncoder()
        result = s.fit_transform(mod1)

        self.assertIsInstance(result, pdml.ModelSeries)
        self.assert_numpy_array_almost_equal(result.values, expected)

        inversed = result.inverse_transform(mod1)
        self.assertIsInstance(inversed, pdml.ModelSeries)
        tm.assert_series_equal(inversed, s)



#!/usr/bin/env python

import sklearn.feature_extraction as fe

import pandas_ml as pdml
import pandas_ml.util.testing as tm


class TestFeatureExtraction(tm.TestCase):

    def test_objectmapper(self):
        df = pdml.ModelFrame([])
        self.assertIs(df.feature_extraction.DictVectorizer, fe.DictVectorizer)
        self.assertIs(df.feature_extraction.FeatureHasher, fe.FeatureHasher)

        self.assertIs(df.feature_extraction.image.img_to_graph, fe.image.img_to_graph)
        self.assertIs(df.feature_extraction.image.grid_to_graph, fe.image.grid_to_graph)
        self.assertIs(df.feature_extraction.image.extract_patches_2d, fe.image.extract_patches_2d)
        self.assertIs(df.feature_extraction.image.reconstruct_from_patches_2d,
                      fe.image.reconstruct_from_patches_2d)
        self.assertIs(df.feature_extraction.image.PatchExtractor, fe.image.PatchExtractor)

        self.assertIs(df.feature_extraction.text.CountVectorizer, fe.text.CountVectorizer)
        self.assertIs(df.feature_extraction.text.HashingVectorizer, fe.text.HashingVectorizer)
        self.assertIs(df.feature_extraction.text.TfidfTransformer, fe.text.TfidfTransformer)
        self.assertIs(df.feature_extraction.text.TfidfVectorizer, fe.text.TfidfVectorizer)



#!/usr/bin/env python


def transform_with_patsy(formula, data, *args, **kwargs):
    try:
        # needs patsy v0.5.1 to support formula in Python 3.7
        # https://github.com/pydata/patsy/pull/131
        import patsy
    except ImportError:
        raise ImportError("'patsy' is required to transform with string formula")

    if '~' in formula:
        y, X = patsy.dmatrices(formula, data=data, return_type='dataframe',
                               *args, **kwargs)
        if len(y.shape) > 1 and y.shape[1] != 1:
            raise ValueError('target must be 1 dimensional')
        y = y.iloc[:, 0]
        return data._constructor(X, target=y)
    else:
        X = patsy.dmatrix(formula, data=data, return_type='dataframe',
                          *args, **kwargs)
        return data._constructor(X)



#!/usr/bin/python
# -*- coding: utf8 -*-

import numpy as np
import pandas as pd
import collections

import pandas_ml as pdml
from pandas_ml.confusion_matrix.stats import binom_interval, class_agreement, prop_test


class ConfusionMatrixAbstract(object):
    """
    Abstract class for confusion matrix

    You shouldn't instantiate this class.
    You might instantiate ConfusionMatrix or BinaryConfusionMatrix classes
    """

    TRUE_NAME = 'Actual'
    PRED_NAME = 'Predicted'

    def __init__(self, y_true, y_pred, labels=None,
                 display_sum=True, backend='matplotlib',
                 true_name='Actual', pred_name='Predicted'):
        self.true_name = true_name
        self.pred_name = pred_name

        if isinstance(y_true, pd.Series):
            self._y_true = y_true
            self._y_true.name = self.true_name
        else:
            self._y_true = pd.Series(y_true, name=self.true_name)

        if isinstance(y_pred, pd.Series):
            self._y_pred = y_pred
            self._y_pred.name = self.pred_name
        else:
            self._y_pred = pd.Series(y_pred, name=self.pred_name)

        if labels is not None:
            if not self.is_binary:
                self._y_true = self._y_true.map(lambda i: self._label(i, labels))
                self._y_pred = self._y_pred.map(lambda i: self._label(i, labels))
            else:
                N = len(labels)
                assert len(labels) == 2, "labels be a list with length=2 - length=%d" % N
                d = {labels[0]: False, labels[1]: True}
                self._y_true = self._y_true.map(d)
                self._y_pred = self._y_pred.map(d)
                raise(NotImplementedError)  # ToDo: see self.classes and BinaryConfusionMatrix.__class ...

        N_true = len(y_true)
        N_pred = len(y_pred)
        assert N_true == N_pred, \
            "y_true must have same size - %d != %d" % (N_true, N_pred)

        df = pd.crosstab(self._y_true, self._y_pred)
        idx = self._classes(df)

        if self.is_binary and pdml.compat._PANDAS_ge_021:
            df = df.reindex([False, True])
            df = df.reindex([False, True], axis=1)
            df = df.fillna(0)
        else:
            df = df.loc[idx, idx.copy()].fillna(0)  # if some columns or rows are missing

        self._df_confusion = df
        self._df_confusion.index.name = self.true_name
        self._df_confusion.columns.name = self.pred_name
        self._df_confusion = self._df_confusion.astype(np.int64)

        self._len = len(idx)

        self.backend = backend
        self.display_sum = display_sum

    def _label(self, i, labels):
        try:
            return(labels[i])
        except IndexError:
            return(i)

    def __repr__(self):
        return(self.to_dataframe(calc_sum=self.display_sum).__repr__())

    def __str__(self):
        return(self.to_dataframe(calc_sum=self.display_sum).__str__())
        # return("%s:\n%s" % (self.title, self.to_dataframe(calc_sum=self.display_sum).__str__()))

    @property
    def classes(self):
        """
        Returns classes (property)
        """
        return(self._classes())

    def _classes(self, df=None):
        """
        Returns classes (method)
        """
        if df is None:
            df = self.to_dataframe()
        idx_classes = (df.columns | df.index).copy()
        idx_classes.name = 'Classes'
        return(idx_classes)

    def to_dataframe(self, normalized=False, calc_sum=False,
                     sum_label='__all__'):
        """
        Returns a Pandas DataFrame
        """
        if normalized:
            a = self._df_confusion.values.astype('float')
            a = a.astype('float') / a.sum(axis=1)[:, np.newaxis]
            df = pd.DataFrame(a,
                              index=self._df_confusion.index.copy(),
                              columns=self._df_confusion.columns.copy())
        else:
            df = self._df_confusion

        if calc_sum:
            df = df.copy()
            df[sum_label] = df.sum(axis=1)
            # df = pd.concat([df, pd.DataFrame(df.sum(axis=1), columns=[sum_label])], axis=1)
            df = pd.concat([df, pd.DataFrame(df.sum(axis=0), columns=[sum_label]).T])
            df.index.name = self.true_name
        return(df)

    @property
    def true(self):
        """
        Returns sum of actual (true) values for each class
        """
        s = self.to_dataframe().sum(axis=1)
        s.name = self.true_name
        return(s)

    @property
    def pred(self):
        """
        Returns sum of predicted values for each class
        """
        s = self.to_dataframe().sum(axis=0)
        s.name = self.pred_name
        return(s)

    def to_array(self, normalized=False, sum=False):
        """
        Returns a Numpy Array
        """
        return(self.to_dataframe(normalized, sum).values)

    def toarray(self, *args, **kwargs):
        """
        see to_array
        """
        return(self.to_array(*args, **kwargs))

    def len(self):
        """
        Returns len of a confusion matrix.
        For example: 3 means that this is a 3x3 (3 rows, 3 columns) matrix
        """
        return(self._len)

    def sum(self):
        """
        Returns sum of a confusion matrix.
        Also called "population"
        It should be the number of elements of either y_true or y_pred
        """
        return(self.to_dataframe().sum().sum())

    @property
    def population(self):
        """
        see also sum
        """
        return(self.sum())

    def y_true(self, func=None):
        if func is None:
            return(self._y_true)
        else:
            return(self._y_true.map(func))

    def y_pred(self, func=None):
        if func is None:
            return(self._y_pred)
        else:
            return(self._y_pred.map(func))

    @property
    def title(self):
        """
        Returns title
        """
        if self.is_binary:
            return("Binary confusion matrix")
        else:
            return("Confusion matrix")

    def plot(self, normalized=False, backend='matplotlib',
             ax=None, max_colors=10, **kwargs):
        """
        Plots confusion matrix
        """

        df = self.to_dataframe(normalized)

        try:
            cmap = kwargs['cmap']
        except KeyError:
            import matplotlib.pyplot as plt
            cmap = plt.cm.gray_r

        title = self.title

        if normalized:
            title += " (normalized)"

        if backend == 'matplotlib':
            import matplotlib.pyplot as plt
            # if ax is None:
            fig, ax = plt.subplots(figsize=(9, 8))
            plt.imshow(df, cmap=cmap, interpolation='nearest')  # imshow / matshow
            ax.set_title(title)

            tick_marks_col = np.arange(len(df.columns))
            tick_marks_idx = tick_marks_col.copy()

            ax.set_yticks(tick_marks_idx)
            ax.set_xticks(tick_marks_col)
            ax.set_xticklabels(df.columns, rotation=45, ha='right')
            ax.set_yticklabels(df.index)

            ax.set_ylabel(df.index.name)
            ax.set_xlabel(df.columns.name)

            # N_min = 0
            N_max = self.max()
            if N_max > max_colors:
                # Continuous colorbar
                plt.colorbar()
            else:
                # Discrete colorbar
                pass
                # ax2 = fig.add_axes([0.93, 0.1, 0.03, 0.8])
                # bounds = np.arange(N_min, N_max + 2, 1)
                # norm = mpl.colors.BoundaryNorm(bounds, cmap.N)
                # cb = mpl.colorbar.ColorbarBase(ax2, cmap=cmap, norm=norm, spacing='proportional', ticks=bounds, boundaries=bounds, format='%1i')

            return ax

        elif backend == 'seaborn':
            import seaborn as sns
            ax = sns.heatmap(df, **kwargs)
            return ax
            # You should test this yourself
            # because I'm facing an issue with Seaborn under Mac OS X (2015-04-26)
            # RuntimeError: Cannot get window extent w/o renderer
            # sns.plt.show()

        else:
            msg = "'backend' must be either 'matplotlib' or 'seaborn'"
            raise ValueError(msg)

    def binarize(self, select):
        """Returns a binary confusion matrix from
        a confusion matrix"""
        if not isinstance(select, collections.Iterable):
            select = np.array(select)

        y_true_bin = self.y_true().map(lambda x: x in select)
        y_pred_bin = self.y_pred().map(lambda x: x in select)

        from pandas_ml.confusion_matrix.bcm import BinaryConfusionMatrix
        binary_cm = BinaryConfusionMatrix(y_true_bin, y_pred_bin)

        return(binary_cm)

    def enlarge(self, select):
        """
        Enlarges confusion matrix with new classes
        It should add empty rows and columns
        """
        if not isinstance(select, collections.Iterable):
            idx_new_cls = pd.Index([select])
        else:
            idx_new_cls = pd.Index(select)
        new_idx = self._df_confusion.index | idx_new_cls
        new_idx.name = self.true_name
        new_col = self._df_confusion.columns | idx_new_cls
        new_col.name = self.pred_name
        print(new_col)
        self._df_confusion = self._df_confusion.loc[:, new_col]
        # self._df_confusion = self._df_confusion.loc[new_idx, new_col].fillna(0)
        # ToFix: KeyError: 'the label [True] is not in the [index]'

    @property
    def stats_overall(self):
        """
        Returns an OrderedDict with overall statistics
        """
        df = self._df_confusion
        d_stats = collections.OrderedDict()

        d_class_agreement = class_agreement(df)

        key = 'Accuracy'
        try:
            d_stats[key] = d_class_agreement['diag']  # 0.35
        except KeyError:
            d_stats[key] = np.nan

        key = '95% CI'
        try:
            d_stats[key] = binom_interval(np.sum(np.diag(df)), df.sum().sum())  # (0.1539, 0.5922)
        except:   # noqa
            d_stats[key] = np.nan

        d_prop_test = prop_test(df)
        d_stats['No Information Rate'] = 'ToDo'  # 0.8
        d_stats['P-Value [Acc > NIR]'] = d_prop_test['p.value']  # 1
        d_stats['Kappa'] = d_class_agreement['kappa']  # 0.078
        d_stats['Mcnemar\'s Test P-Value'] = 'ToDo'  # np.nan

        return(d_stats)

    @property
    def stats_class(self):
        """
        Returns a DataFrame with class statistics
        """
        # stats = ['TN', 'FP', 'FN', 'TP']
        # df = pd.DataFrame(columns=self.classes, index=stats)
        df = pd.DataFrame(columns=self.classes)

        # ToDo Avoid these for loops

        for cls in self.classes:
            binary_cm = self.binarize(cls)
            binary_cm_stats = binary_cm.stats()
            for key, value in binary_cm_stats.items():
                df.loc[key, cls] = value  # binary_cm_stats

        d_name = {
            'population': 'Population',
            'P': 'P: Condition positive',
            'N': 'N: Condition negative',
            'PositiveTest': 'Test outcome positive',
            'NegativeTest': 'Test outcome negative',
            'TP': 'TP: True Positive',
            'TN': 'TN: True Negative',
            'FP': 'FP: False Positive',
            'FN': 'FN: False Negative',
            'TPR': 'TPR: (Sensitivity, hit rate, recall)',  # True Positive Rate
            'TNR': 'TNR=SPC: (Specificity)',  # True Negative Rate
            'PPV': 'PPV: Pos Pred Value (Precision)',
            'NPV': 'NPV: Neg Pred Value',
            'prevalence': 'Prevalence',
            # 'xxx': 'xxx: Detection Rate',
            # 'xxx': 'xxx: Detection Prevalence',
            # 'xxx': 'xxx: Balanced Accuracy',
            'FPR': 'FPR: False-out',
            'FDR': 'FDR: False Discovery Rate',
            'FNR': 'FNR: Miss Rate',
            'ACC': 'ACC: Accuracy',
            'F1_score': 'F1 score',
            'MCC': 'MCC: Matthews correlation coefficient',
            'informedness': 'Informedness',
            'markedness': 'Markedness',
            'LRP': 'LR+: Positive likelihood ratio',
            'LRN': 'LR-: Negative likelihood ratio',
            'DOR': 'DOR: Diagnostic odds ratio',
            'FOR': 'FOR: False omission rate',
        }
        df.index = df.index.map(lambda id: self._name_from_dict(id, d_name))

        return(df)

    def stats(self, lst_stats=None):
        """
        Return an OrderedDict with statistics
        """
        d_stats = collections.OrderedDict()
        d_stats['cm'] = self
        d_stats['overall'] = self.stats_overall
        d_stats['class'] = self.stats_class
        return(d_stats)

    def _name_from_dict(self, key, d_name):
        """
        Returns name (value in dict d_name
        or key if key doesn't exists in d_name)
        """
        try:
            return(d_name[key])
        except (KeyError, TypeError):
            return(key)

    def _str_dict(self, d, line_feed_key_val='\n',
                  line_feed_stats='\n\n', d_name=None):
        """
        Return a string representation of a dictionary
        """
        s = ""
        for i, (key, val) in enumerate(d.items()):
            name = self._name_from_dict(key, d_name)
            if i != 0:
                s = s + line_feed_stats
            s = s + "%s:%s%s" % (name, line_feed_key_val, val)
        return(s)

    def _str_stats(self, lst_stats=None):
        """
        Returns a string representation of statistics
        """
        d_stats_name = {
            "cm": "Confusion Matrix",
            "overall": "Overall Statistics",
            "class": "Class Statistics",
        }

        stats = self.stats(lst_stats)

        d_stats_str = collections.OrderedDict([
            ("cm", str(stats['cm'])),
            ("overall", self._str_dict(
                stats['overall'],
                line_feed_key_val=' ', line_feed_stats='\n')),
            ("class", str(stats['class'])),
        ])

        s = self._str_dict(
            d_stats_str, line_feed_key_val='\n\n',
            line_feed_stats='\n\n\n', d_name=d_stats_name)
        return(s)

    def print_stats(self, lst_stats=None):
        """
        Prints statistics
        """
        print(self._str_stats(lst_stats))

    def get(self, actual=None, predicted=None):
        """
        Get confusion matrix value for a given
        actual class and a given predicted class

        if only one parameter is given (actual or predicted)
        we get confusion matrix value for actual=actual and predicted=actual
        """
        if actual is None:
            actual = predicted
        if predicted is None:
            predicted = actual
        return(self.to_dataframe().loc[actual, predicted])

    def max(self):
        """
        Returns max value of confusion matrix
        """
        return(self.to_dataframe().max().max())

    def min(self):
        """
        Returns min value of confusion matrix
        """
        return(self.to_dataframe().min().min())

    @property
    def is_binary(self):
        """Return False"""
        return(False)

    @property
    def classification_report(self):
        """
        Returns a DataFrame with classification report
        """
        columns = np.array(['precision', 'recall', 'F1_score', 'support'])
        index = self.classes
        df = pd.DataFrame(index=index, columns=columns)

        for cls in self.classes:
            binary_cm = self.binarize(cls)
            for stat in columns:
                df.loc[cls, stat] = getattr(binary_cm, stat)

        total_support = df.support.sum()
        df.loc['__avg / total__', :] = (df[df.columns[:-1]].transpose() * df.support).sum(axis=1) / df.support.sum()
        df.loc['__avg / total__', 'support'] = total_support

        return(df)

    def _avg_stat(self, stat):
        """
        Binarizes confusion matrix
        and returns (weighted) average statistics
        """
        s_values = pd.Series(index=self.classes)
        for cls in self.classes:
            binary_cm = self.binarize(cls)
            v = getattr(binary_cm, stat)
            print(v)
            s_values[cls] = v
        value = (s_values * self.true).sum() / self.population
        return(value)



#!/usr/bin/env python

from pandas_ml.core.accessor import _AccessorMethods


def _to_dmatrix(data):
    import xgboost as xgb
    dm = xgb.DMatrix(data.data, label=data.target)
    return dm


class XGBoostMethods(_AccessorMethods):
    """Accessor to ``xgboost``."""

    # cannot attach classes automatically, because the module doesn't have __all__
    _module_name = 'xgboost.sklearn'

    @property
    def XGBRegressor(self):
        import xgboost as xgb
        return xgb.XGBRegressor

    @property
    def XGBClassifier(self):
        import xgboost as xgb
        return xgb.XGBClassifier

    def plot_importance(self, ax=None, height=0.2,
                        xlim=None, title='Feature importance',
                        xlabel='F score', ylabel='Features',
                        grid=True, **kwargs):

        """Plot importance based on fitted trees.

        Parameters
        ----------
        ax : matplotlib Axes, default None
            Target axes instance. If None, new figure and axes will be created.
        height : float, default 0.2
            Bar height, passed to ax.barh()
        xlim : tuple, default None
            Tuple passed to axes.xlim()
        title : str, default "Feature importance"
            Axes title. To disable, pass None.
        xlabel : str, default "F score"
            X axis title label. To disable, pass None.
        ylabel : str, default "Features"
            Y axis title label. To disable, pass None.
        kwargs :
            Other keywords passed to ax.barh()

        Returns
        -------
        ax : matplotlib Axes
        """

        import xgboost as xgb

        if not isinstance(self._df.estimator, xgb.XGBModel):
            raise ValueError('estimator must be XGBRegressor or XGBClassifier')
        # print(type(self._df.estimator.booster), self._df.estimator.booster)
        return xgb.plot_importance(self._df.estimator,
                                   ax=ax, height=height, xlim=xlim, title=title,
                                   xlabel=xlabel, ylabel=ylabel, grid=True, **kwargs)

    def to_graphviz(self, num_trees=0, rankdir='UT',
                    yes_color='#0000FF', no_color='#FF0000', **kwargs):

        """Convert specified tree to graphviz instance. IPython can automatically plot the
        returned graphiz instance. Otherwise, you shoud call .render() method
        of the returned graphiz instance.

        Parameters
        ----------
        num_trees : int, default 0
            Specify the ordinal number of target tree
        rankdir : str, default "UT"
            Passed to graphiz via graph_attr
        yes_color : str, default '#0000FF'
            Edge color when meets the node condigion.
        no_color : str, default '#FF0000'
            Edge color when doesn't meet the node condigion.
        kwargs :
            Other keywords passed to graphviz graph_attr

        Returns
        -------
        ax : matplotlib Axes
        """

        import xgboost as xgb

        if not isinstance(self._df.estimator, xgb.XGBModel):
            raise ValueError('estimator must be XGBRegressor or XGBClassifier')
        return xgb.to_graphviz(self._df.estimator,
                               num_trees=num_trees, rankdir=rankdir,
                               yes_color=yes_color, no_color=no_color, **kwargs)

    def plot_tree(self, num_trees=0, rankdir='UT', ax=None, **kwargs):

        """Plot specified tree.

        Parameters
        ----------
        booster : Booster, XGBModel
            Booster or XGBModel instance
        num_trees : int, default 0
            Specify the ordinal number of target tree
        rankdir : str, default "UT"
            Passed to graphiz via graph_attr
        ax : matplotlib Axes, default None
            Target axes instance. If None, new figure and axes will be created.
        kwargs :
            Other keywords passed to to_graphviz

        Returns
        -------
        ax : matplotlib Axes

        """

        import xgboost as xgb

        if not isinstance(self._df.estimator, xgb.XGBModel):
            raise ValueError('estimator must be XGBRegressor or XGBClassifier')
        return xgb.plot_tree(self._df.estimator,
                             num_trees=num_trees, rankdir=rankdir, **kwargs)



#!/usr/bin/env python

from pandas_ml.compat import cache_readonly
from pandas_ml.core.accessor import _AccessorMethods


class FeatureExtractionMethods(_AccessorMethods):
    """
    Accessor to ``sklearn.feature_extraction``.
    """

    _module_name = 'sklearn.feature_extraction'

    # image

    @property
    def image(self):
        """Property to access ``sklearn.feature_extraction.image``"""
        return self._image

    @cache_readonly
    def _image(self):
        return _AccessorMethods(self._df, module_name='sklearn.feature_extraction.image')

    # text

    @property
    def text(self):
        """Property to access ``sklearn.feature_extraction.text``"""
        return self._text

    @cache_readonly
    def _text(self):
        attrs = ['CountVectorizer', 'HashingVectorizer',
                 'TfidfTransformer', 'TfidfVectorizer']
        return _AccessorMethods(self._df, module_name='sklearn.feature_extraction.text',
                                attrs=attrs)
